<?php

/**
 * The plugin options management class
 *
 * @link       http://themify.me
 * @since      1.0.0
 *
 * @package    PTB
 * @subpackage PTB/includes
 */

/**
 * The plugin options helper class
 *
 *
 * @package    PTB
 * @subpackage PTB/includes
 * @author     Themify <ptb@themify.me>
 */
class PTB_Submissiion_Options {

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $plugin_name The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private $version;

    /**
     * Custom Post Types Templates array
     *
     * @since    1.0.0
     * @access   private
     * @var      array $option_post_type_templates The options of custom post types templates.
     */

    /**
     * The plugin options array
     *
     * @since    1.0.0
     * @access   private
     * @var      array $options The options of the plugin.
     */
    private $options;
    private $settings_key = 'options';
    private $options_key_settings = 'settings';
    protected $option_settings = array();

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @var      string $plugin_name The name of this plugin.
     * @var      string $version The version of this plugin.
     */
    public function __construct($plugin_name, $version) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->settings_key = $this->plugin_name . '-' . $this->settings_key;
        $this->load_options();
    }

    /**
     * Loads the plugin options.
     * Default options created if plugin options are empty
     *
     * @since 1.0.0
     */
    private function load_options() {

        $this->options = get_option($this->settings_key);
        if (empty($this->options)) {
            $this->options = $this->get_options_blueprint();
        }

        $this->option_settings = &$this->options[$this->options_key_settings];
    }

    private function get_options_blueprint() {
        return array(
            $this->options_key_settings => array()
        );
    }

    public function get_settings() {
        return $this->option_settings;
    }

    public function set_settings($settings) {
        $this->option_settings = $settings;
    }

    public function update() {
        return update_option($this->settings_key, $this->options);
    }

    public function get_submission_template($post_type) {
        $ptb_options = PTB::get_option();
        foreach ($ptb_options->option_post_type_templates as $k => $t) {
            if (isset($t['frontend']) && isset($t['frontend']['data']) && isset($t['post_type']) && $post_type == $t['post_type']) {
                $t['id'] = $k;
                return $t;
            }
        }
        return FALSE;
    }

    public static function get_name($type, $multi = false) {
        $names = array(
            'editor' => __('Content', 'ptb-submission'),
            'title' => __('Title', 'ptb-submission'),
            'thumbnail' => __('Featured Image', 'ptb-submission'),
            'taxonomies' => $multi ? __('Taxonomies', 'ptb-submission') : __('Taxonomy', 'ptb-submission'),
            'category' => $multi ? __('Categories', 'ptb-submission') : __('Category', 'ptb-submission'),
            'excerpt' => __('Excerpt', 'ptb-submission'),
            'user_name' => __('Username', 'ptb-submission'),
            'user_email' => __('Email Address', 'ptb-submission'),
            'user_password' => __('Password', 'ptb-submission'),
            'user_confirm_password' => __('Confirm Password', 'ptb-submission')
        );
        return isset($names[$type]) ? $names[$type] : FALSE;
    }

    /**
     * Get full list of currency codes.
     * @return array
     */
    public function get_currencies() {
        return array_unique(
                apply_filters('ptb_submission_currencies', array(
            'AED' => __('United Arab Emirates Dirham', 'ptb-submission'),
            'ARS' => __('Argentine Peso', 'ptb-submission'),
            'AUD' => __('Australian Dollars', 'ptb-submission'),
            'BDT' => __('Bangladeshi Taka', 'ptb-submission'),
            'BRL' => __('Brazilian Real', 'ptb-submission'),
            'BGN' => __('Bulgarian Lev', 'ptb-submission'),
            'CAD' => __('Canadian Dollars', 'ptb-submission'),
            'CLP' => __('Chilean Peso', 'ptb-submission'),
            'CNY' => __('Chinese Yuan', 'ptb-submission'),
            'COP' => __('Colombian Peso', 'ptb-submission'),
            'CZK' => __('Czech Koruna', 'ptb-submission'),
            'DKK' => __('Danish Krone', 'ptb-submission'),
            'DOP' => __('Dominican Peso', 'ptb-submission'),
            'EUR' => __('Euros', 'ptb-submission'),
            'HKD' => __('Hong Kong Dollar', 'ptb-submission'),
            'HRK' => __('Croatia kuna', 'ptb-submission'),
            'HUF' => __('Hungarian Forint', 'ptb-submission'),
            'ISK' => __('Icelandic krona', 'ptb-submission'),
            'IDR' => __('Indonesia Rupiah', 'ptb-submission'),
            'INR' => __('Indian Rupee', 'ptb-submission'),
            'NPR' => __('Nepali Rupee', 'ptb-submission'),
            'ILS' => __('Israeli Shekel', 'ptb-submission'),
            'JPY' => __('Japanese Yen', 'ptb-submission'),
            'KIP' => __('Lao Kip', 'ptb-submission'),
            'KRW' => __('South Korean Won', 'ptb-submission'),
            'MYR' => __('Malaysian Ringgits', 'ptb-submission'),
            'MXN' => __('Mexican Peso', 'ptb-submission'),
            'NGN' => __('Nigerian Naira', 'ptb-submission'),
            'NOK' => __('Norwegian Krone', 'ptb-submission'),
            'NZD' => __('New Zealand Dollar', 'ptb-submission'),
            'PYG' => __('Paraguayan Guaraní', 'ptb-submission'),
            'PHP' => __('Philippine Pesos', 'ptb-submission'),
            'PLN' => __('Polish Zloty', 'ptb-submission'),
            'GBP' => __('Pounds Sterling', 'ptb-submission'),
            'RON' => __('Romanian Leu', 'ptb-submission'),
            'RUB' => __('Russian Ruble', 'ptb-submission'),
            'SGD' => __('Singapore Dollar', 'ptb-submission'),
            'ZAR' => __('South African rand', 'ptb-submission'),
            'SEK' => __('Swedish Krona', 'ptb-submission'),
            'CHF' => __('Swiss Franc', 'ptb-submission'),
            'TWD' => __('Taiwan New Dollars', 'ptb-submission'),
            'THB' => __('Thai Baht', 'ptb-submission'),
            'TRY' => __('Turkish Lira', 'ptb-submission'),
            'UAH' => __('Ukrainian Hryvnia', 'ptb-submission'),
            'USD' => __('US Dollars', 'ptb-submission'),
            'VND' => __('Vietnamese Dong', 'ptb-submission'),
            'EGP' => __('Egyptian Pound', 'ptb-submission')
                        )
                )
        );
    }

    /**
     * Get Currency symbol.
     * @param string $currency
     * @return string
     */
    public function get_currency_symbol($currency) {

        switch ($currency) {
            case 'AED' :
                $currency_symbol = 'د.إ';
                break;
            case 'AUD' :
            case 'ARS' :
            case 'CAD' :
            case 'CLP' :
            case 'COP' :
            case 'HKD' :
            case 'MXN' :
            case 'NZD' :
            case 'SGD' :
            case 'USD' :
                $currency_symbol = '&#36;';
                break;
            case 'BDT':
                $currency_symbol = '&#2547;&nbsp;';
                break;
            case 'BGN' :
                $currency_symbol = '&#1083;&#1074;.';
                break;
            case 'BRL' :
                $currency_symbol = '&#82;&#36;';
                break;
            case 'CHF' :
                $currency_symbol = '&#67;&#72;&#70;';
                break;
            case 'CNY' :
            case 'JPY' :
            case 'RMB' :
                $currency_symbol = '&yen;';
                break;
            case 'CZK' :
                $currency_symbol = '&#75;&#269;';
                break;
            case 'DKK' :
                $currency_symbol = 'DKK';
                break;
            case 'DOP' :
                $currency_symbol = 'RD&#36;';
                break;
            case 'EGP' :
                $currency_symbol = 'EGP';
                break;
            case 'EUR' :
                $currency_symbol = '&euro;';
                break;
            case 'GBP' :
                $currency_symbol = '&pound;';
                break;
            case 'HRK' :
                $currency_symbol = 'Kn';
                break;
            case 'HUF' :
                $currency_symbol = '&#70;&#116;';
                break;
            case 'IDR' :
                $currency_symbol = 'Rp';
                break;
            case 'ILS' :
                $currency_symbol = '&#8362;';
                break;
            case 'INR' :
                $currency_symbol = 'Rs.';
                break;
            case 'ISK' :
                $currency_symbol = 'Kr.';
                break;
            case 'KIP' :
                $currency_symbol = '&#8365;';
                break;
            case 'KRW' :
                $currency_symbol = '&#8361;';
                break;
            case 'MYR' :
                $currency_symbol = '&#82;&#77;';
                break;
            case 'NGN' :
                $currency_symbol = '&#8358;';
                break;
            case 'NOK' :
                $currency_symbol = '&#107;&#114;';
                break;
            case 'NPR' :
                $currency_symbol = 'Rs.';
                break;
            case 'PHP' :
                $currency_symbol = '&#8369;';
                break;
            case 'PLN' :
                $currency_symbol = '&#122;&#322;';
                break;
            case 'PYG' :
                $currency_symbol = '&#8370;';
                break;
            case 'RON' :
                $currency_symbol = 'lei';
                break;
            case 'RUB' :
                $currency_symbol = '&#1088;&#1091;&#1073;.';
                break;
            case 'SEK' :
                $currency_symbol = '&#107;&#114;';
                break;
            case 'THB' :
                $currency_symbol = '&#3647;';
                break;
            case 'TRY' :
                $currency_symbol = '&#8378;';
                break;
            case 'TWD' :
                $currency_symbol = '&#78;&#84;&#36;';
                break;
            case 'UAH' :
                $currency_symbol = '&#8372;';
                break;
            case 'VND' :
                $currency_symbol = '&#8363;';
                break;
            case 'ZAR' :
                $currency_symbol = '&#82;';
                break;
            default :
                $currency_symbol = '';
                break;
        }

        return apply_filters('ptb_submission_currency_symbol', $currency_symbol, $currency);
    }

    /**
     * Get full list of currency codes.
     * @return array
     */
    public function get_currency_position() {
        return array_unique(
                apply_filters('ptb_submission_currency_position', array(
            'left' => __('Left (£99.99)', 'ptb-submission'),
            'right' => __('Right (99.99£)', 'ptb-submission'),
            'left_space' => __('Left with space (£ 99.99)', 'ptb-submission'),
            'right_space' => __('Right with space (99.99 £)', 'ptb-submission')
                        )
                )
        );
    }

    /**
     * Get the price format depending on the currency position
     *
     * @return string
     */
    function get_price_format($currency_pos, $currency, $price) {
        $format = '%1$s%2$s';
        if (!$currency_pos) {
            $get_options = get_option($this->plugin_name . '-settings');
            $currency_pos = $get_options['currency_position'];
        }
        switch ($currency_pos) {
            case 'left' :
                $format = '%1$s%2$s';
                break;
            case 'right' :
                $format = '%2$s%1$s';
                break;
            case 'left_space' :
                $format = '%1$s&nbsp;%2$s';
                break;
            case 'right_space' :
                $format = '%2$s&nbsp;%1$s';
                break;
        }
        $format = apply_filters('ptb_submission_price_format', $format, $currency, $currency_pos);
        return sprintf($format, $this->get_currency_symbol($currency), $price);
    }

    public static function max_upload_size($size) {
        $max_upload = wp_max_upload_size();
        if (isset($size) || !$size) {
            $size = floatval($size);
            if ($size < 1 || $size > $max_upload) {
                $size = $max_upload;
            }
        } else {
            $size = $max_upload;
        }
        return $size;
    }

    public static function get_allow_ext(array $extension = array(), $type = 'image') {
        switch ($type) {
            case 'video':
                $ext = array('wmv', 'avi', 'flv', 'mov|qt', 'mpeg|mpg|mpe', 'mp4|m4v', 'ogv', 'webm');
                break;
            case 'audio':
                $ext = array('mp3|m4a|m4b', 'ogg|oga', 'wav', 'wma');
                break;
            default:
                $ext = array('gif', 'png', 'bmp', 'jpg|jpeg|jpe', 'tif|tiff', 'ico');
        }
        $allowed_extensions = get_allowed_mime_types();
        $can_be_allowed = !empty($extension) && !in_array('all', $extension) ? $extension : $ext;
        $allow = array();
        foreach ($can_be_allowed as $k => &$e) {
            if (isset($allowed_extensions[$e])) {
                $allow[$e] = $allowed_extensions[$e];
            }
        }
        return $allow;
    }

    public static function HumanFileSize($size, $unit = false) {
        if ((!$unit && $size >= 1 << 30) || $unit == "GB")
            return number_format($size / (1 << 30), 2) . __("GB", 'ptb-submission');
        if ((!$unit && $size >= 1 << 20) || $unit == "MB")
            return number_format($size / (1 << 20), 2) . __("MB", 'ptb-submission');
        if ((!$unit && $size >= 1 << 10) || $unit == "KB")
            return number_format($size / (1 << 10), 2) . __("KB", 'ptb-submission');
        return number_format($size) . __(" bytes", 'ptb-submission');
    }

    public static function FileUniqname($dir, $filename, $ext) {
        $filename = sanitize_file_name($filename);
        $name = preg_match('/[^A-Za-z0-9-_]$/iu', $filename) ? md5($filename) : $filename;
        return wp_unique_filename($dir, $name . $ext);
    }

    public static function validate_file(array $f, array $allow, $size = NULL) {

        $error = false;
        $size = self::max_upload_size($size);
        if ($f['size'] > $size) {
            $error = sprintf(__("Maximum upload file size is %s", 'ptb-submission'), self::HumanFileSize($size));
        } else {
            if (!function_exists('wp_handle_upload')) {
                require_once( ABSPATH . 'wp-admin/includes/file.php' );
            }
            $movefile = wp_handle_upload($f, array('test_form' => false, 'mimes' => $allow, 'unique_filename_callback' => array(__CLASS__, 'FileUniqname')));
            if ($movefile && !isset($movefile['error'])) {
                $movefile['name'] = $f['name'];
                $movefile['size'] = $f['size'];
            } else {
                $error = $movefile['error'];
            }
        }
        @unlink($f['tmp_name']);
        return $error ? array('error' => $error) : array('file' => $movefile);
    }

}
