<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 * Base class for metaboxes
 *
 * @author am
 */
class PTB_Submission {

    /**
     * The ID of plugin.
     *
     * @since    1.0.0
     * @access   public
     * @var      string $plugin_name The ID of this plugin.
     */
    private static $plugin_name;

    /**
     * The version of plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $version The current version of this plugin.
     */
    private static $version;

    /**
     * The type of custom meta box.
     *
     * @since    1.0.0
     * @access   private
     * @var      string $type The type of custom meta box.
     */
    private static $options;

    public function __construct($version) {

        self::$plugin_name = 'ptb-submission';
        self::$version = $version;
        self::load_dependencies();
        self::set_locale();
        if (is_admin()) {
            $this->define_admin_hooks();
        }
        if (!is_admin() || (defined('DOING_AJAX') && DOING_AJAX)) {
            $this->define_public_hooks();
        }
    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - PTB_Loader. Orchestrates the hooks of the plugin.
     * - PTB_i18n. Defines internationalization functionality.
     * - PTB_Admin. Defines all hooks for the dashboard.
     * - PTB_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private static function load_dependencies() {
        $plugin_dir = plugin_dir_path(dirname(__FILE__));

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once $plugin_dir . 'includes/class-ptb-submissions-i18n.php';
        require_once $plugin_dir . 'includes/class-ptb-submissions-options.php';
        require_once $plugin_dir . 'includes/class-ptb-submissions-captcha.php';
        if (is_admin()) {
            require_once $plugin_dir . 'includes/class-ptb-submissions-list.php';
            require_once $plugin_dir . 'includes/class-ptb-submissions-posts.php';
            require_once $plugin_dir . 'includes/class-ptb-submissions-users.php';
            require_once $plugin_dir . 'admin/class-ptb-submissions-admin.php';
            require_once $plugin_dir . 'includes/class-ptb-form-ptt-frontend.php';
        }
        if (!is_admin() || (defined('DOING_AJAX') && DOING_AJAX)) {
            require_once $plugin_dir . 'includes/class-ptb-submissions-payment.php';
            require_once $plugin_dir . 'includes/class-ptb-submissions-paypal.php';
            require_once $plugin_dir . 'public/class-ptb-submissions-public.php';
        }


        // The classes of metaboxes

        do_action('ptb_submission_loaded');
    }

    private function define_admin_hooks() {
        $options = PTB::get_option();
        $plugin_admin = new PTB_Submission_Admin(self::$plugin_name, self::$version, $options);
        add_action('admin_init', array($plugin_admin, 'init'));
        add_action('admin_menu', array($plugin_admin, 'add_plugin_admin_menu'));
        add_action('before_template_row', array($plugin_admin, 'add_row_required'), 10, 4);
        add_action('admin_enqueue_scripts', array($plugin_admin, 'enqueue_scripts'));
        add_action('ptb_cpt_update', array($plugin_admin, 'cpt_update'), 10, 2);
        add_action('ptb_cpt_remove', array($plugin_admin, 'cpt_remove'), 10, 1);
        add_action('ptb_deactivated', array($plugin_admin, 'deactivate'));
        add_action('wp_ajax_ptb_submission_add', array($plugin_admin, 'add_template'));
        add_action('wp_ajax_ptb_submission_edit', array($plugin_admin, 'edit_template'));
        add_action('wp_ajax_save_ajax', array($plugin_admin, 'save_temlate'));
        add_action('wp_ajax_ptb_submission_get_import', array($plugin_admin, 'import_temlate'));
        add_action('wp_ajax_ptb_submission_import', array($plugin_admin, 'set_import'));
        add_action('wp_ajax_ptb_submission_delete', array($plugin_admin, 'delete'));
        add_action('wp_ajax_ptb_submission_list', array($plugin_admin, 'get_list'));
        add_action('wp_ajax_ptb_submission_posts_filter', array($plugin_admin, 'posts_lists'));
        add_action('wp_ajax_ptb_submission_post_action', array($plugin_admin, 'post_action'));
        add_filter('ptb_shorcode_template_menu', array($plugin_admin, 'shortcode_menu'));
        add_filter('ptb_ajax_shortcode_result', array($plugin_admin, 'ptb_shortcode_result'), 10, 2);
        add_filter('ptb_template_modules', array($plugin_admin, 'ptb_submission_modules'), 10, 3);
        add_filter('ptb_template_save', array($plugin_admin, 'ptb_template_save'), 10, 2);
        add_filter('ptb_screens', array($plugin_admin, 'screens'), 10, 2);
    }

    private function define_public_hooks() {
        $plugin_public = new PTB_Submission_Public(self::$plugin_name, self::$version);
        //add_action('wp_enqueue_scripts', array($plugin_public,'enqueue_scripts'));
        add_action('init', array($plugin_public, 'register_session'));
        if (!is_admin()) {
            add_filter('the_title', array($plugin_public, 'page_title'), 10, 2);
            add_filter('wp_title', array($plugin_public, 'wp_title'), 10, 2);
            add_filter('author_link', array($plugin_public, 'author_url'), 10, 3);
            //  add_action( 'pre_get_posts', array($plugin_public,'author_posts'),99999,1 );
        }
        add_filter('script_loader_tag', array($plugin_public, 'ptb_ignore_rocket_loader'));
        add_shortcode('ptb_submission', array($plugin_public, 'shortcode'));
        add_shortcode('ptb_submission_account', array($plugin_public, 'account'));
        add_action('wp_ajax_ptb_submission_terms', array($plugin_public, 'get_terms'));
        add_action('wp_ajax_ptb_account_filter', array($plugin_public, 'posts_filter'));
        add_action('wp_ajax_ptb_account_remove_post', array($plugin_public, 'remove_post'));
        add_action('wp_ajax_ptb_submission_save', array($plugin_public, 'save_post'));
        add_action('wp_ajax_nopriv_ptb_submission_save', array($plugin_public, 'save_post'));
        add_action('wp_ajax_ptb_submission_profile', array($plugin_public, 'edit_profile'));
        add_action('wp_ajax_nopriv_ptb_submission_payment', array($plugin_public, 'payment_form'));
        add_action('wp_ajax_ptb_submission_payment', array($plugin_public, 'payment_form'));
        add_action('wp_ajax_nopriv_ptb_submission_payment_result', array($plugin_public, 'payment_result'));
        add_action('wp_ajax_ptb_submission_payment_result', array($plugin_public, 'payment_result'));
    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the PTB_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private static function set_locale() {

        $plugin_i18n = new PTB_Submission_i18n();
        $plugin_i18n->set_domain(self::$plugin_name);
        $plugin_i18n->load_plugin_textdomain();
    }

    public static function get_option() {
        if (!isset(self::$options)) {
            self::$options = new PTB_Submissiion_Options(self::$plugin_name, self::$version);
        }
        return self::$options;
    }

}
