(function ($) {
    'use strict';
    var filesList = {},
        paramNames = {};
    var InitLanguageTabs = function () {
        $(document).on('click', '.ptb_language_tabs li', function (e) {
            e.preventDefault();
            if (!$(this).hasClass('ptb_active_tab_lng')) {
                var $tab = $(this).closest('ul');
                var $fields = $tab.next('ul');
                $tab.find('.ptb_active_tab_lng').removeClass('ptb_active_tab_lng');
                $(this).addClass('ptb_active_tab_lng');
                $fields.find('.ptb_active_lng').removeClass('ptb_active_lng');
                $fields.find('li').eq($(this).index()).addClass('ptb_active_lng');
            }
        });
    };

    function bytesToSize(bytes) {
        var sizes = ['n/a', 'B', 'KB', 'MB'];
        var i = +Math.floor(Math.log(bytes) / Math.log(1024));
        return  (bytes / Math.pow(1024, i)).toFixed(i ? 1 : 0) + ' ' + sizes[ isNaN(bytes) ? 0 : i + 1 ];
    }
    var InitFileUpload = function ($files) {

        $files.each(function () {
            var $parent_node = $(this).closest('.ptb-submission-text-option');
            $parent_node = $parent_node.length > 0 ? $parent_node : $(this).closest('.ptb_module');
            var $form = $(this).closest('form'),
                    $allow_extension = $(this).data('extension') ? new RegExp('(\\.|\/)(' + $(this).data('extension') + ')$', 'i') : /(\.|\/)(gif|jpe?g|png)$/i,
                    $max_file_size = $(this).data('size'),
                    $width = $(this).data('width'),
                    $height = $(this).data('height'),
                    node = $parent_node.find('.ptb-submission-priview'),
                    $this = $(this);
            $(this).fileupload({
                url: $form.attr('action'),
                type: 'POST',
                acceptFileTypes: $allow_extension,
                maxFileSize: $max_file_size,
                dataType: 'json',
                singleFileUploads: true,
                autoUpload: false,
                disableImageResize: /Android(?!.*Chrome)|Opera/.test(window.navigator.userAgent),
                previewMaxWidth: $width ? $width : 100,
                previewMaxHeight: $height ? $height : 100,
                previewCrop: true
            }).on('fileuploadprocessalways', function (e, data) {
                var index = data.index,
                        file = data.files[index],
                        $fname = $('.ptb-submission-upload-btn[for="' + $this.attr('id') + '"');
                        node.next('.ptb-submission-file-danger').remove();
                        node.hide('blind', 500, function () {
                    if (file.error) {
                        $fname.html($this.data('label'));
                        $(this).after($('<span class="ptb-submission-file-danger ptb-submission-error-text"/>').text(file.error));
                    }
                    else {
                        if (file.preview) {
                            $(this).html(file.preview);
                            if ($(this).closest('.ptb-submission-multi-text').length === 0 && $(this).closest('.ptb-submission-module-req').length === 0) {
                                $(this).append('<span data-slug="' + $this.attr('id') + '" class="fa fa-minus-circle ptb-submission-file-remove"></span>');
                            }
                            $(this).append('<span class="ptb-submission-file-name">' + file.name + '</span>')
                                    .append('<span class="ptb-submission-file-size">' + bytesToSize(file.size) + '</span>').width(data.previewMaxWidth).show('blind', 500);
                        }
                        else {
                            $(this).html('');
                        }

                        var $f = file.name.length > 12 ? file.name.substring(0, 12) + '...' : file.name;
                        $fname.html('<span>' + $f + '</span>');
                    }
                });
                if (!file.error) {
                    for (var i = 0; i < data.files.length; i++) {
                        var $key = $(this).attr('id');
                        filesList[$key] = data.files[i];
                        paramNames[$key] = e.delegateTarget.name;
                    }
                }
            });

        });
    };

    var InitFileRemove = function () {
        $('.ptb_module').delegate('.ptb-submission-file-remove', 'click', function (e) {
            e.preventDefault();
            var $preview = $(this).closest('.ptb-submission-priview');
            var $fwrap = $preview.find('.ptb-submission-file-wrap'),
                $slug = $(this).data('slug');
            FileRemove($fwrap, $slug, $preview);
        });
    };

    var FileRemove = function ($fwrap, $key, $preview) {
        $preview.hide('blind', 500, function () {
            $(this).empty();
        });
        delete paramNames[$key];
        delete filesList[$key];
        $fwrap.html($fwrap.html());
        var $label = $('.ptb-submission-upload-btn[for="' + $key + '"');
        $label.html($label.data('label'));
        InitFileUpload($fwrap.find('.ptb-submission-file'));
    }

    var InitAutoComplete = function () {
        var $autocomplete = $('.ptb-submission-autocomplete');
        var $cache = [];
        $autocomplete.each(function () {
            var $slug = $(this).data('slug'),
                    $this = $(this),
                    $post_type = $(this).data('post_type');
            $cache[$slug] = {};
            $(this).autocomplete({
                minLength: 2,
                source: function (request, response) {
                    var term = request.term;
                    if (term in $cache[$slug]) {
                        response($cache[$slug][ term ]);
                        return;
                    }
                    $.ajax({
                        url: ajaxurl,
                        dataType: 'json',
                        type: 'POST',
                        data: {
                            term: request.term,
                            slug: $slug,
                            post_type: $post_type,
                            action: 'ptb_submission_terms'
                        },
                        success: function (data) {
                            $cache[$slug][ term ] = data;
                            response(data);
                        }
                    });
                },
                focus: function (event, ui) {
                    event.preventDefault();
                },
                open: function () {
                    $("ul.ui-menu").width($(this).innerWidth());
                },
                select: function (event, ui) {
                    $this.val(ui.item.label);
                    $this.next('input').val(ui.item.value);
                    return false;
                }
            });
        });
    };

    var InitMultiText = function () {
        var $multitext = $('.ptb-submission-text-option input,.ptb-submission-text-option textarea');
        $multitext.each(function () {
            var $name = $(this).attr('name');
            if ($name) {
                $(this).attr('name', $name + '[]');
            }
        });
        var $fields = $('.ptb-submission-multi-text li');
        $fields.each(function () {
            if ($(this).find('.ptb-submission-priview').length > 0) {
                var $id = MakeUniqueId();
                $(this).find('label').prop('for', $id);
                $(this).find('input[type="file"]').prop('id', $id);
            }
        });
        $fields = $multitext = null;
        $('.ptb-submission-option-add').click(function (e) {
            e.preventDefault();
            e.stopPropagation();
            var $module = $(this).closest('.ptb-submission-multi-text');
            var $li = $module.find('.ptb-submission-text-option').first().clone(),
                    $wrapper = $module.children('ul');
            $li.find('input,textarea').val('');
            $li.find('.ptb-submission-required-error').removeClass('ptb-submission-required-error');
            var $preview = $li.find('.ptb-submission-priview');
            $.event.trigger("ptb_submission_before_option_add", $li);
            if ($preview.length > 0) {
                var $id = MakeUniqueId();
                $li.find('.ptb-submission-error-text').remove();
                var $label = $li.find('label');
                $label.prop('for', $id);
                $label.html($label.data('label'));
                $li.find('input[type="file"]').prop('id', $id);
                $preview.empty().hide();
            }
            $li.appendTo($wrapper).hide().show('blind', 500);
            if ($preview.length > 0) {
                InitFileUpload($('#' + $id));
            }
            $.event.trigger("ptb_submission_after_option_add", $li);
            $wrapper.sortable("refresh");
        });
        $('.ptb-submission-form').delegate('.ptb-submission-remove', 'click', function (e) {
            e.preventDefault();
            e.stopPropagation();
            var $option = $(this).closest('.ptb-submission-text-option');
            var $preview = $option.find('.ptb-submission-priview');
            if ($preview.length > 0) {
                var $fwrap = $preview.closest('.ptb-submission-text-option').find('.ptb-submission-file-wrap'),
                    $slug = $fwrap.find('input').attr('id');
                FileRemove($fwrap, $slug, $preview);
                if ($option.closest('ul').children('li').length == 1) {
                    $option.find('input').val('');
                    return false;
                }
            }
            $option.hide('blind', 500, function () {
                $(this).remove();
            });
        });
        $('.ptb-submission-multi-text>ul').sortable({
            placeholder: "ui-state-highlight"
        });
    };

    var MakeUniqueId = function () {
        return 'ptb_submission_' + Math.random().toString(36).substr(2, 9);
    };

    var InitSelect = function () {
        $('.ptb-select').chosen({
            disable_search_threshold: 7,
            search_contains: true,
            width: "100%"
        });
    };

    var InitCaptchaRefresh = function () {

        $('.ptb-submission-captcha-refresh').click(function (e) {
            e.preventDefault();
            var $form = $(this).closest('form');
            $form.addClass('ptb-submission-wait');
            var $img = $(this).closest('table').find('img');
            $img.attr('src', $img.attr('src')).load(function () {
                $form.removeClass('ptb-submission-wait');
            });
        });
    };

    var InitSubmit = function () {
        $('.ptb-submission-form').on('submit', function (e) {
            e.preventDefault();
            var $form = $(this);
            var $validate = true;
            var $req = $form.find('.ptb-submission-module-req'),
                $error = $form.find('.ptb-submission-form-error');
            $form.find('.ptb-submission-required-error,.ptb-submission-label-required').removeClass('ptb-submission-required-error ptb-submission-label-required');
            $form.find('.switch-html').trigger('click');
            $req.each(function () {
                var $label = $(this);
                var $input = $label.find('input,select,textarea');
                $input.each(function () {
                    var $er = false,
                            $name = $(this).attr('name');
                    if (!$name) {
                        return false;
                    }
                    if ($(this).is(':checkbox') || $(this).is(':radio')) {
                        if ($label.find('[name^="' + $name + '"]:checked').length > 0) {
                            return false;
                        }
                        $er = true;
                    }
                    else if (!$.trim($(this).val()) && $(this).closest('.chosen-container').length == 0) {
                        if ($.trim($(this).closest('.ptb_module').find('.ptb-submission-priview').html())) {
                            return false;
                        }
                        $er = true;
                    }
                    var $type = $(this).closest('.ptb_module').data('type');
                    var $check = $.event.trigger("ptb_submission_validate_"+$type,[$(this),filesList]);
                    if (typeof $check !== 'undefined') {
                        $er = $check;
                    }
                    if ($er) {
                        $(this).addClass('ptb-submission-required-error');
                        $label.addClass('ptb-submission-label-required');
                        $validate = false;
                    }
                });
            });
            if ($validate) {

                $error.html('');
                if (Object.keys(filesList).length > 0) {
                    var $filesList = [],
                        $paramNames = [];
                    var $multi = $('.ptb-submission-multi-text');
                    if ($multi.length > 0) {
                        var $li = $multi.find('>ul>li');
                        $li.each(function () {
                            var $inputs = $(this).find('input,textarea');
                            var $index = $(this).index();
                            $inputs.each(function () {
                                var $name = $(this).attr('name');
                                if ($name) {
                                    $name = $name.replace(/\[[0-9]*\]/i, '[' + $index + ']');
                                    $(this).attr('name', $name);
                                }
                            });
                        });
                    }
                    for (var $f in filesList) {
                        if ($('#' + $f).closest('.ptb-submission-multi-text').length > 0) {
                            var $index = $('#' + $f).closest('li').index();
                            paramNames[$f] = paramNames[$f].replace(/\[[0-9]*\]/i, '[' + $index + ']');
                        }
                        $paramNames.push(paramNames[$f]);
                        $filesList.push(filesList[$f]);
                    }
                    $form.addClass('ptb-submission-wait');
                    $('.ptb-submission-file', $form).fileupload('send',
                            {files: $filesList,
                                paramName: $paramNames,
                            }).complete(function () {
                        $form.removeClass('ptb-submission-wait');
                    }).success(function (resp) {
                        SubmitResult(resp, $form);
                    });
                }
                else {
                    $.ajax({
                        url: $form.attr('action'),
                        type: 'POST',
                        dataType: 'json',
                        data: $form.serialize(),
                        beforeSend: function () {
                            $form.addClass('ptb-submission-wait');
                        },
                        complete: function () {
                            $form.removeClass('ptb-submission-wait');
                        },
                        success: function (resp) {
                            SubmitResult(resp, $form);
                        }
                    });
                }
            }
            else {
                $error.html(ptb_submission.errors);
            }
        });
    };

    var SubmitResult = function (resp, $form) {
        var $errors = $form.find('.ptb-submission-errors'),
                $error = $form.find('.ptb-submission-form-error');
        $errors.html('');
        $form.find('.ptb-submission-error-text').remove();
        if (resp) {
            if (resp.success) {
                if (resp.r) {
                    window.location.href = resp.success;
                }
                else {
                    var $href = window.location.search ? window.location.href + '&' : window.location.href + '?';
                    $href += 'nonce=' + resp.nonce + '&form_id=' + resp.form_id;
                    window.location.href = $href;
                }
            }
            else if (resp.fee) {
                window.location.href = resp.fee;
            }
            else {

                for (var $i in resp) {

                    if ($i == 'captcha') {
                        $form.find('.ptb-submission-captcha-refresh').trigger('click');
                    }
                    if (typeof resp[$i] !== 'object') {
                        var $message = resp[$i];
                        resp[$i] = [];
                        resp[$i][0] = 1;
                    }
                    for (var $j in resp[$i]) {
                        var $input = $form.find('[name^="submission[' + $i + ']"]'),
                                $text = $j != 0 ? resp[$i][$j] : $message;

                        if ($input.length == 0) {
                            $input = $form.find('[name^="' + $i + '"]');
                        }
                        if ($input.length > 0) {
                            $input.addClass('ptb-submission-required-error');
                            $input.closest('.ptb_module').addClass('ptb-submission-label-required');
                            $input.first().closest('.ptb_back_active_module_input').append('<div class="ptb-submission-error-text">' + $text + '</div>');
                        }
                        else {
                            $errors.append('<div>' + $text + '</div>');
                        }
                    }
                }
                $error.html(ptb_submission.errors);
            }
        }
    };

    var InitResetForm = function ($form) {
        if (!$form.find('[name="id"]').val()) {
            var $multy = $form.find('.ptb-submission-multi-text');
            $multy.each(function () {
                $(this).find('.ptb-submission-text-option:not(:first)').remove();
            });
            $form.find('input[type="text"],input[type="email"],textarea').val('');
            var $select = $form.find('.ptb-select');
            $select.find('option:selected').removeAttr('selected');
            $select.trigger("chosen:updated");
            filesList = paramNames = [];
            $form.find('.ptb-submission-priview').html('').hide();
            var $autocomplete = $form.find('.ptb-submission-autocomplete');
            $autocomplete.each(function () {
                $(this).closest('div').find('input[type="hidden"]').val('');
            });
        }
    };


    $(document).ready(function () {
        InitLanguageTabs();
        InitFileUpload($('input[type="file"].ptb-submission-file'));
        InitFileRemove();
        InitAutoComplete();
        InitSelect();
        InitMultiText();
        InitCaptchaRefresh();
        InitSubmit();
        $('form.ptb-submission-form').each(function () {
            var $preview = $(this).find('.ptb-submission-priview');
            $preview.each(function () {
                if ($(this).children('*').length > 0) {
                    $(this).show();
                }
            });
        });
    });

}(jQuery));